import configparser

import click
from rich.console import Console
from rich.table import Table
from rich.tree import Tree

from .selection import fzf
from .utils import exit_with_msg, path_is_valid_subkt

console = Console()


@click.group()
@click.help_option("--help", "-h")
def config() -> None:
    """Add, remove, edit projects in the config."""
    pass


@config.command()
@click.pass_context
def add(ctx: click.Context) -> None:
    """
    Add projects to config interactively.

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        None
    """

    config = ctx.obj["config"]

    while True:
        click.clear()

        project_name = click.prompt("Name of the project", type=str)

        if " " in project_name:
            console.print("Space in project name is not valid.")
            if not click.confirm("Do you want to add another project?", default=False):
                break
            continue

        while True:
            project_path = click.prompt("Path to the project", type=click.Path())

            if path_is_valid_subkt(project_path):
                break

        # Ask for folder structure type
        give_folder_structure_info()
        folder_structure = click.prompt(
            "Choose folder structure (normal/alternate)",
            type=click.Choice(["normal", "alternate"], case_sensitive=False),
        )

        exceptions = {}
        if folder_structure == "alternate":
            console.print(
                "Add exceptions when folder name does not match what you put in sub.properties.\n"
                "For more info on exceptions, read alternate folder section [underline][link=https://github.com/PhosCity/muxkt]in this link[/link][/underline]!"
            )

            while True:
                exception_key = click.prompt(
                    "Enter exception key (or press Enter to finish)",
                    type=str,
                    default="",
                )
                if not exception_key.strip():  # Exit the loop if input is blank
                    break
                exception_value = click.prompt(
                    f"Enter exception value'{exception_key}'", type=str, default=""
                )
                if not exception_value.strip():  # Handle blank values
                    console.print(
                        f"[red]Error:[/red] Value for '{exception_key}' cannot be blank."
                    )
                    continue
                exceptions[exception_key] = exception_value

        config = add_to_configparser(config, "Project", {project_name: project_path})
        config = add_to_configparser(
            config, "Folder Structure", {project_name: folder_structure}
        )
        if folder_structure == "alternate":
            config = add_to_configparser(
                config, f"{project_name}_exceptions", exceptions
            )

        if not click.confirm("Do you want to add another project?", default=False):
            break

    save_config(config, ctx.obj["config_file"])


@config.command()
@click.pass_context
def remove(ctx: click.Context) -> None:
    """
    Remove a project from the config.

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        None
    """

    config = ctx.obj["config"]

    if not config.has_section("Project"):
        exit_with_msg("No projects exist in the configuration.")

    project_list = config.options("Project")
    project_to_remove = fzf(
        project_list, "Select project to remove", choose_multiple=False
    )
    if not project_to_remove:
        exit_with_msg("No project selected for removal.")

    config.remove_option("Project", project_to_remove)
    save_config(config, ctx.obj["config_file"])


@config.command()
@click.pass_context
def edit_manual(ctx: click.Context) -> None:
    """
    Manually edit the config file using the default editor on your computer.

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        None
    """

    config_file = ctx.obj["config_file"]

    try:
        click.edit(filename=config_file)
    except click.exceptions.ClickException as e:
        exit_with_msg(f"Could not open editor: {e}")
    except Exception as e:
        exit_with_msg(str(e))


@config.command()
@click.pass_context
def edit(ctx: click.Context) -> None:
    """
    Edit the config file interactively.

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        None
    """

    config = ctx.obj["config"]

    sections = config.sections()
    section = fzf(sections, "Select Section to Edit", choose_multiple=False)

    if not section:
        exit_with_msg("No section selected for editing.")

    def edit_key_value(section, key, value_type=str):
        """Edit a single key-value pair interactively."""
        new_key = click.prompt(
            f"Change name of the key (current: {key})",
            type=str,
            default="",
            show_default=False,
        )
        if not new_key:
            console.print("[cyan]No changes made to the key name.[/cyan]")
            new_key = key

        new_value = click.prompt(
            f"Change value for '{new_key}' (current: {config.get(section, key)})",
            type=value_type,
            default="",
            show_default=False,
        )
        if not new_value:
            console.print("[cyan]No changes made to the value.[/cyan]")
            new_value = config.get(section, key)

        return new_key, new_value

    if section == "Folder Structure":
        key = fzf(
            config.options(section),
            "Choose project you want to edit the folder structure of.",
            choose_multiple=False,
        )
        folder_structure_choice = fzf(
            ["normal", "alternate"],
            f"Change folder structure for '{key}' (current: {config.get("Folder Structure", key)})",
            choose_multiple=False,
        )

        if folder_structure_choice:
            config.set("Folder Structure", key, folder_structure_choice)
    elif section == "Project":
        keys = fzf(
            config.options(section),
            "Choose all keys you want to edit using tab key",
            choose_multiple=True,
        )
        for key in keys:
            while True:
                new_key, new_value = edit_key_value(
                    section, key, value_type=click.Path()
                )
                if path_is_valid_subkt(new_value):
                    config.remove_option(section, key)
                    config.set(section, new_key, new_value)
                    break
    else:
        keys = fzf(
            config.options(section),
            "Choose all keys you want to edit using tab key",
            choose_multiple=True,
        )

        for key in keys:
            new_key, new_value = edit_key_value(section, key)
            config.remove_option(section, key)
            config.set(section, new_key, new_value)

    save_config(config, ctx.obj["config_file"])


def add_history(
    ctx: click.Context,
    project: str,
    path: str,
    episode: list,
    custom_flag: tuple,
) -> None:
    """
    Replaces the 'History' section with new project, path and episodes.

    Args:
        ctx (click.Context): Context passed by click from the entry point.
        project (str): The name of the project.
        path (str): The path associated with the project.
        episode (list): The list of episodes.
        custom_flag (tuple): The custom flags for the muxing command.

    Returns:
        None
    """

    config = ctx.obj["config"]

    if config.has_section("History"):
        config.remove_section("History")

    config.add_section("History")

    serialized_episode = ",".join(str(x) for x in episode)
    serialized_flags = ",".join(str(x) for x in custom_flag)

    config.set("History", "project", project)
    config.set("History", "path", path)
    config.set("History", "episode", serialized_episode)
    config.set("History", "custom_flags", serialized_flags)

    save_config(config, ctx.obj["config_file"])


def get_history(
    ctx: click.Context,
) -> tuple[str, str, list[str], list[str]]:
    """
    Retrieve the history from the config file.

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        str: Name of the project muxed last time.
        str: Path of the project muxed last time.
        list: Episodes of the project that were muxed last time.
        list: Custom flags for the last mux.
    """

    config = ctx.obj["config"]

    if not config.has_section("History"):
        exit_with_msg("No history found in the configuration.")

    project = config.get("History", "project")
    path = config.get("History", "path")

    serialized_episode = config.get("History", "episode")
    episode = serialized_episode.split(",")

    serialized_flags = config.get("History", "custom_flags")
    flags = serialized_flags.split(",") if serialized_flags else []

    return project, path, episode, flags


def read_config(
    config: configparser.ConfigParser,
    project: str | None,
):
    """
    Read the config and collects the project's name and its path.

    Args:
        config (configparser.ConfigParser): The configuration object.
        project (str | None): Name of the project whose path to find in config; None if user wants to select project and its path interactively from config.

    Returns:
        str: Name of the project muxed last time.
        str: Path of the project muxed last time.
    """

    if not config.has_section("Project"):
        exit_with_msg(
            "No projects found in the config. Run 'muxkt config add' to add projects."
        )

    if not project:
        project = fzf(
            config.options("Project"),
            "Select a project",
            choose_multiple=False,
        )
        if not project:
            exit_with_msg("No project selected.")

    try:
        path = config.get("Project", project)
        return project, path
    except configparser.NoOptionError:
        exit_with_msg(f"No project named '{project}' found in the config")


def add_to_configparser(
    config: configparser.ConfigParser,
    section_name: str,
    section_options: dict,
) -> configparser.ConfigParser:
    """
    Add sections to the config object

    Args:
        config (configparser.ConfigParser): The configuration object.
        section_name (str): Name of the section to add to the config.
        section_options(dict): Options to add to the config

    Returns:
        config (configparser.ConfigParser): The configuration object.
    """

    if not config.has_section(f"{section_name}"):
        config.add_section(f"{section_name}")

    for key, value in section_options.items():
        config.set(f"{section_name}", key, value)

    return config


def save_config(
    config: configparser.ConfigParser,
    config_file: str,
) -> None:
    """
    Saves the current configuration to the config file.

    Args:
        config (configparser.ConfigParser): The configuration object.
        config_file (str): Path of the config file

    Returns:
        None
    """

    with open(config_file, "w") as c:
        config.write(c)


def give_folder_structure_info() -> None:
    """
    Prints the information about folder structures

    Args:
        None

    Returns:
        None
    """

    normal = Tree(".")
    for item in ["01", "02", "03", "...", "Subkt Configs"]:
        normal.add(item)

    alternate = Tree(".")
    arc_1 = alternate.add("01 Name of Arc 1/ Season 1")
    for item in ["01", "02", "..."]:
        arc_1.add(item)

    arc_2 = alternate.add("01 Name of Arc 1/ Season 1")
    for item in ["01", "02", "..."]:
        arc_2.add(item)

    alternate.add("Subkt Configs")

    table = Table(show_lines=True)

    table.add_column("Folder Type", justify="left", no_wrap=True)
    table.add_column("Folder Tree", justify="left", no_wrap=True)

    table.add_row("normal", normal)
    table.add_row("alternate", alternate)
    console.print(table)
    console.print(
        "For more info on folder structure, read alternate folder section [underline][link=https://github.com/PhosCity/muxkt]in this link[/link][/underline]!"
    )



from time import sleep

from iterfzf import iterfzf


def selection(iterable: list):
    """
    Takes in a list and allows the user to choose among the list using fzf

    Args:
        iterable (list): The list from which user should choose an option

    Return:
        Generator Function for iterfzf
    """

    for item in iterable:
        yield item.strip()
        sleep(0.01)


def fzf(iterable: list, prompt: str, choose_multiple: bool = False):
    """
    Takes in a list and allows the user to choose among the list using fzf

    Args:
        iterable (list): The list from which user should choose an option
        prompt (str): Prompt to show in the fzf interface
        choose_multiple (bool): True if user can choose multiple options; False if user cannot

    Return:
        list | str: list of chosen items if 'choose_multiple' is true; string of chosen item otherwise
    """

    return iterfzf(selection(iterable), prompt=prompt, multi=choose_multiple)



import os
import sys
from shutil import which

from rich import box
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

console = Console()


def path_is_valid_subkt(path: str) -> bool:
    """
    Checks if the given path contains a file named 'build.gradle.kts'.

    Args:
        path (str): The path to check.

    Returns:
        bool: True if 'build.gradle.kts' exists in the path, False otherwise.
    """

    if not os.path.isdir(path):
        console.print(
            f"[red]Error:[/red] The provided path '{path}' is not a valid directory."
        )
        return False

    if not os.path.isfile(os.path.join(path, "build.gradle.kts")):
        console.print(
            f"[red]Error:[/red] The provided path '{path}' does not appear to be a valid path that has subkt files."
        )
        return False
    return True


def exit_with_msg(message: str) -> None:
    """
    Exit the program with a styled error message.

    Args:
        message (str): Message to show before exiting.

    Returns:
        None
    """

    console.print(f"[bold red]Error:[/bold red] {message}")
    sys.exit(1)


def check_dependencies() -> bool:
    """
    Check if required dependencies are installed and accessible.

    Returns:
        bool: Returns True if all dependencies are met; otherwise exits the program after showing the list of dependencies that are not installed.
    """

    dependencies = ["java", "mkvmerge"]
    missing_dependencies = [dep for dep in dependencies if which(dep) is None]

    if missing_dependencies:
        console.print("[bold red]The following dependencies are missing:[/bold red]")

        # Display missing dependencies in a table
        table = Table(show_header=True, header_style="bold magenta")
        table.add_column("No.", style="dim", width=4)
        table.add_column("Dependency", style="bold")

        for i, dep in enumerate(missing_dependencies, start=1):
            table.add_row(str(i), dep)

        console.print(table)
        sys.exit(1)
    else:
        return True


def msg_in_box(title: str, message: str | Text) -> None:
    """
    Print text in a box with title

    Args:
        title (str): Title of the box
        message (str): Message to be shown inside the box

    Returns:
        None
    """

    console.print(
        Panel.fit(
            message,
            box=box.ROUNDED,
            padding=(1, 2),
            title=title,
            border_style="bright_blue",
            title_align="left",
        ),
    )



import configparser
import os

import click
from rich.traceback import install

from .config import config
from .mux import mux

install(show_locals=True, suppress=[click])


@click.group()
@click.version_option()
@click.help_option("--help", "-h")
@click.pass_context
def cli(ctx: click.Context) -> None:
    config_file_path = click.get_app_dir("muxkt")

    if not os.path.exists(config_file_path):
        os.makedirs(config_file_path)

    config_file = os.path.join(config_file_path, "config")
    output_file = os.path.join(config_file_path, "output.txt")

    config = configparser.ConfigParser()

    if not os.path.exists(config_file):
        with open(config_file, "w") as c:
            config.write(c)

    config.read(config_file)

    ctx.obj = {
        "config_file_path": config_file_path,
        "config_file": config_file,
        "config": config,
        "output_file": output_file,
    }


cli.add_command(config)
cli.add_command(mux)



import os
import re
import subprocess
import sys

import click
from rich.console import Console
from rich.table import Table
from rich.text import Text

from .config import add_history, get_history, read_config
from .selection import fzf
from .utils import check_dependencies, exit_with_msg, msg_in_box

console = Console()


@click.command()
@click.pass_context
@click.help_option("--help", "-h")
@click.argument(
    "project",
    required=False,
    nargs=1,
)
@click.argument(
    "episode",
    required=False,
    nargs=-1,
    type=int,
)
@click.option(
    "-r",
    "--repeat",
    is_flag=True,
    help="Repeat last muxing action.",
)
@click.option(
    "-o",
    "--output",
    is_flag=True,
    help="See original output of previous mux",
)
@click.option(
    "-c",
    "--custom_flag",
    type=str,
    multiple=True,
    help="Provide multiple custom Gradle flags (e.g., -Pkey=value).",
)
def mux(
    ctx: click.Context,
    project: str | None,
    episode: tuple,
    repeat: bool,
    output: bool,
    custom_flag: tuple,
) -> None:
    """Mux the episodes using the arguments and the options provided by the user."""
    """
    Args:
        ctx (click.Context): Context passed by click from the entry point.
        project (str | None): Name of the project. None if no argument provided.
        episode (tuple | None): Tuple of episodes that user provided as an argument; empty if no argument provided.

    Options:
        repeat_last (bool): Mux using last mux settings. True if user used --repeat or --r option; otherwise False
        output (bool): Show output of last mux verbatim and exit. True if user used --output or --o; otherwise False
        custom_flag (str): Custom flag that user wants to append to the gradle command

    Returns:
        None
    """

    if output:
        cat_output(ctx)

    check_dependencies()

    if repeat:
        project_name, path, episode, custom_flag = get_history(ctx)
    else:
        project_name, path, episode = get_project_info(ctx, project, episode)

    add_history(ctx, project_name, path, episode, custom_flag)

    output_file = ctx.obj["output_file"]

    try:
        os.chdir(path)
    except FileNotFoundError:
        exit_with_msg(f"The path '{path}' does not exist.")
    except PermissionError:
        exit_with_msg(f"You do not have permission to access '{path}'.")

    click.clear()
    for ep in episode:
        cmdfile = "./gradlew" if os.name == "posix" else "gradlew.bat"
        command = [cmdfile, "--console=plain"]
        if custom_flag:
            command.extend(custom_flag)
        command.append(f"mux.{ep}")

        try:
            with console.status(f'[cyan]Muxing "{project_name}" - Episode {ep}[/cyan]'):
                with open(output_file, "w") as f:
                    result = subprocess.run(command, stdout=f, stderr=f, text=True)

            console.print(f'[cyan]Muxing "{project_name}" - Episode {ep}[/cyan]')
            if result.returncode == 0:
                mux_success(output_file)
            else:
                mux_warning(output_file)
                mux_failure(output_file)

            console.rule()

        except Exception as e:
            exit_with_msg(f"Error during muxing: {e}")


def mux_success(output_file: str) -> None:
    """
    Process the muxing outpt file and display categorized results.

    Args:
        output_file (str): The path to the file where output of the mux is stored.

    Returns:
        None
    """

    try:
        with open(output_file, "r") as f:
            lines = f.read()
    except FileNotFoundError:
        exit_with_msg("Output file not found.")

    patterns_and_headers = [
        (r"> Task :([^S].*)", "TASKS PERFORMED:"),
        (r"(CHAPTER.*)", "CHAPTERS GENERATED:"),
        (r"(Track.*])", "TRACK LIST:"),
        (r"Attaching (.*[otOT][tT][fF])", "FONTS ATTACHED:"),
        (r"(Validating fonts.*|warning: .*)", "WARNINGS:"),
        (r"Attaching (.*[otOT][tT][fF])", "DUPLICATE FONTS ATTACHED:"),
        (r"Output: (.*mkv)", "OUTPUT:"),
        (r"(\d+ actionable tasks:.*)", ""),
        (r"(BUILD SUCCESSFUL in .*s)", ""),
    ]

    for pattern, header in patterns_and_headers:
        matches = [match.group(1) for match in re.finditer(pattern, lines)]
        if not matches:
            continue

        # Process results based on the header
        if header == "WARNINGS:":
            mux_warning(output_file)
            continue

        elif header == "TASKS PERFORMED:":
            console.rule(Text(header, style="bold green"))
            matches = [match.replace(".default", "") for match in matches]

            table = Table(row_styles=["dim", "none"])
            table.add_column(style="dim")

            for i, item in enumerate(matches):
                match = re.compile(r"([^.]+)\.([^\s]+)( UP-TO-DATE)?").match(item)
                if not match:
                    continue

                number_padded = str(i + 1)
                name_part = match.group(1)
                after_dot_part = match.group(2)
                status = match.group(3) or "EXECUTED"
                if i == 0:  # Set the header only once
                    table.add_column(f"Task performed for {after_dot_part}")
                    table.add_column("Status")
                table.add_row(number_padded, name_part, status.strip())

            console.print(table)

        elif header == "FONTS ATTACHED:":
            console.rule(Text(header, style="bold green"))
            matches.sort()
            table = Table(show_header=False, row_styles=["dim", "none"])

            for index, font in enumerate(matches, start=1):
                table.add_row(str(index), font)

            console.print(table)

        elif header == "DUPLICATE FONTS ATTACHED:":
            matches = list(set(item for item in matches if matches.count(item) > 1))
            if not matches:
                continue

            console.rule(Text(header, style="bold green"))
            table = Table(show_header=False, row_styles=["dim", "none"])

            for index, font in enumerate(matches, start=1):
                table.add_row(str(index), font)

            console.print(table)

        elif header == "TRACK LIST:":
            console.rule(Text(header, style="bold green"))
            table = Table(row_styles=["dim", "none"])
            table.add_column("Track")
            table.add_column("Metadata")
            table.add_column("File")

            pattern = r"Track (\w+) \((.*?)\) \[(.*?)\]$"

            for item in matches:
                match = re.search(pattern, item)
                if match:
                    table.add_row(match.group(1), match.group(2), match.group(3))

            console.print(table)

        elif header == "CHAPTERS GENERATED:":
            console.rule(Text(header, style="bold green"))
            table = Table(row_styles=["dim", "none"])
            table.add_column("Name", justify="left")
            table.add_column("Timestamp", justify="left")

            for i in range(0, len(matches), 2):
                line1 = matches[i]
                line2 = matches[i + 1]

                _, value1 = line1.split("=")
                _, value2 = line2.split("=")

                table.add_row(value2, value1)
            console.print(table)

        else:
            if header:
                console.rule(Text(header, style="bold green"))
            for match in matches:
                click.echo(match)
        console.print()


def mux_warning(output_file: str) -> None:
    """
    Process the muxing outpt file and display the warnings found in the file.
    Warnings include font validation messages and general warning messages.

    Args:
        output_file (str): The path to the file where output of the mux is stored.

    Returns:
        None
    """

    try:
        with open(output_file, "r") as f:
            lines = f.readlines()
    except FileNotFoundError:
        exit_with_msg("Output file not found.")

    # Try to group warnings for each subtitle separately
    grouped = []
    current_group = []
    for line in lines:
        if re.search(r"[vV]alidating fonts for.*", line):
            if current_group:
                grouped.append(current_group)
            current_group = [line.strip()[:-3]]
        elif re.search(r"[wW]arning: .*", line):
            warning = re.sub(r"^.*[wW]arning: (.*).*$", r"\1", line)
            if current_group:
                current_group.append(warning.strip())

    if current_group:
        grouped.append(current_group)

    # Bail out early if there are not warnings collected.
    if not grouped:
        return

    # Print warnings
    console.rule(Text("WARNINGS:", style="bold green"))
    for group in grouped:
        title = group.pop(0)

        text = Text()
        if not group:
            group.append("No issues were found.")

        for txt in group:
            style = "bold magenta" if "not found" in txt else None
            text.append(txt, style=style)
            text.append("\n")

        msg_in_box(title, text)
        console.print()


def mux_failure(output_file: str) -> None:
    """
    Process the muxing outpt file and display the reasons why the mux failed.

    Args:
        output_file (str): The path to the file where output of the mux is stored.

    Returns:
        None: Prints out the formatted mux output.
    """

    failure_patterns = [
        r"(FAILURE: .*)",
        r"(.*What went wrong.*)",
        r"(A problem occurred.*)",
        r"(Execution failed for task.*)",
        r"(Error resolving.*)",
        r"(.*not found in root project.*)",
        r"(style already exists.*)",
        r"(one or more fatal font-related issues encountered.*)",
        r"(FileNotFoundException.*)",
        r"(mkvmerge -J command failed.*)",
        r"(mkvmerge -J command timed out for file.*)",
        r"(malformed property.*)",
        r"(mkvmerge failed:.*)",
        r"(Error: .*)",
        r"(is ambiguous in root project.*)",
        r"(.*could not find target sync line.*)",
        r"(could not find property file.*)",
        r"(Could not create task.*)",
        r"(no chapter definitions found;.*)",
        r"(Negative time after shifting line from.*)",
        r"(Could not resolve.*)",
        r"(Could not list available versions.*)",
        r"(duplicate target sync lines with value.*)",
        r"(could not post to webhook:.*)",
        r"(Unexpected CRC for.*)",
        r"(not a valid CRC:.*)",
        r"(malformed line in.*)",
        r"(Recursive property dependency detected:.*)",
        r"(Attempting to access unfinished task.*)",
        r"(Attempted to access entry.*)",
        r"(more than one file added, but no root set, or conflicting roots..*)",
        r"(couldn't upload torrent:.*)",
        r"(request failed:.*)",
        r"(could not upload.*)",
        r"(can't convert type to destination directory:.*)",
        r"(Invalid SSL Session.*)",
        r"(Could not create directory:.*)",
        r"(ssh command failed.*)",
        r"(no conversion available from String to.*)",
        r"(Invalid value for Collisions:.*)",
        r"(too few fields in section.*)",
        r"(could not parse.*)",
        r"(no match for property name.*)",
        r"(not a valid time:.*)",
        r"(not a valid color:.*)",
        r"(not a valid boolean:.*)",
        r"(not a valid boolean:.*)",
        r"(BUILD FAILED.*)",
    ]

    with open(output_file, "r") as f:
        lines = f.read()

    for pattern in failure_patterns:
        matches = re.finditer(pattern, lines)
        for match in matches:
            console.print(match.group(1))
            console.print()

    line_list = lines.splitlines()
    # Print subkt compilaton errors
    start_line = "Script compilation errors:"
    end_line = r"^\d+ errors$"

    start_index = -1
    end_index = -1

    if start_line in line_list:
        start_index = line_list.index(start_line)

    for i, line in enumerate(line_list):
        if re.match(end_line, line):
            end_index = i
            break

    if start_index != -1 and end_index != -1 and start_index < end_index:
        msg_in_box(
            "Script compilaton errors",
            "\n".join(line_list[start_index + 1 : end_index + 1]),
        )


def cat_output(ctx: click.Context) -> None:
    """
    Print out the actual subkt output of previous mux

    Args:
        ctx (click.Context): Context passed by click from the entry point.

    Returns:
        None: Prints out the previous output and exits the program.
    """
    output_file = ctx.obj["output_file"]

    try:
        with open(output_file, "r") as f:
            content = f.read()

        console.print("[bold cyan]Output File Content:[/bold cyan]")
        click.echo(content)
        sys.exit(0)
    except FileNotFoundError:
        exit_with_msg(f"File not found: {output_file}")
    except IOError as e:
        exit_with_msg(f"Could not read output file: {e}")


def get_project_info(
    ctx: click.Context,
    project: str | None,
    episode: tuple,
) -> tuple[str, str, list[str]]:
    """
    Gets the info about the project.

    Args:
        project (str | None): Project argument given by user; None if no argument provided.
        episode (tuple): Tuple of episodes that user provided as an argument; empty if no argument provided.

    Returns:
        project_name (str): Name of the project either from history or from config
        path (str): Path of the project either from history or from config
        episode (list[str]): List of all the episodes that needs to be muxed.
    """

    project_name, path = read_config(ctx.obj["config"], project)

    config = ctx.obj["config"]

    alternate_folder = config.get("Folder Structure", project_name) == "alternate"

    if episode:
        episode = [f"{ep:02}" for ep in sorted(episode)]

    if episode and not alternate_folder:
        return project_name, path, episode

    elif not episode and not alternate_folder:
        return (
            project_name,
            path,
            select_folder(path, "Select single or multiple episode: ", True),
        )

    else:
        arc = select_folder(path, "Select an arc/season: ", False)
        console.print(arc)
        if not episode:
            episode = select_folder(
                os.path.join(path, arc),
                "Select single or multiple episode: ",
                True,
            )
        arc = arc[3:].replace(" ", "").lower()

        exceptions_section = f"{project_name}_exceptions"
        if config.has_option(exceptions_section, arc):
            arc = config.get(exceptions_section, arc)

        return project_name, path, [arc + "_" + str(ep) for ep in episode]


def select_folder(
    path: str,
    prompt: str,
    multi: bool,
) -> list[str] | str:
    """
    Allows the user to select folders within the given path that start with a number.

    Args:
        path (str): The directory path to search for folders.
        prompt (str): The prompt message to display to the user.
        multi (bool): Whether to allow multiple selections.

    Returns:
        list or str: The selected folder(s). Returns a list if `multi` is True; otherwise, a single folder name.
    """

    # Get a sorted list of directories in the given path that start with a digit
    valid_folders = sorted(
        folder
        for folder in os.listdir(path)
        if os.path.isdir(os.path.join(path, folder)) and folder[0].isdigit()
    )
    selected_folders = fzf(valid_folders, prompt=prompt, choose_multiple=multi)

    if not selected_folders:
        exit_with_msg("User did not select anything. Exiting.")
    return selected_folders
